class = require("Lib.30log")
room  = require("Room")
require("TileMap")
require("MapTile")
require("EnemySpawner")
require("ItemGenerator")


MapLoader = class("MapLoader", {pixel_width, pixel_height, tile_width, tile_height, items_to_place})
function MapLoader:init(pixel_width, pixel_height, tile_width, tile_height, items_to_place)
  
  self.map_pixels = {}
    self.map_pixels.w = pixel_width + 240
    self.map_pixels.h = pixel_height
  
  self.tile_size = {}
    self.tile_size.w = tile_width
    self.tile_size.h = tile_height
    
  self.map_tiles = {}
    self.map_tiles.w = (self.map_pixels.w - 240) / self.tile_size.w
    self.map_tiles.h = self.map_pixels.h / self.tile_size.h
    
  self.tilemap = TileMap:new(self.map_tiles.w, self.map_tiles.h)
  
  self.images = {}
    self.images.ground = love.graphics.newImage("Images/Tiles/tile_ground.png")
    self.images.wall = love.graphics.newImage("Images/Tiles/tile_wall.png")
    self.images.fog_color = {255,255,255,200}
  
  self.room_limits = {}
    self.room_limits.max = {}
      self.room_limits.max.w = 10
      self.room_limits.max.h = 10
    self.room_limits.min = {}
      self.room_limits.min.w = 4
      self.room_limits.min.h = 4
  
  self.enemy_spawner = nil
  self.item_generator = nil
  
  self.current_flr_lvl = 1
  
  self.floor = nil
  
  self.room_limit = self.map_tiles.w
  
  self.rooms = {}
  
  self.current_room = nil
  self.room_count = 0
  self.map_full = false
  
  self.next_pos = 0
end

function MapLoader:create_room()

  self.current_room = Room:new(self.tilemap)
  self.current_room:set_pos(self.next_pos)
  self.current_room:set_size()
  self.current_room:set_center()
                          
  if self.current_room:get_last() < self.map_tiles.w then
    table.insert(self.rooms, self.current_room)
    self.room_count = self.room_count + 1
    self.next_pos = self.current_room.center.x
  else
    self.map_full = true
  end
end

function MapLoader:get_room(index)
  return self.rooms[index]
end

function MapLoader:get_roompos(index)
  return self.rooms[index].pos
end

function MapLoader:get_roomsize(index)
  return self.rooms[index].size
end

function MapLoader:get_roomcenter(index)
  return self.rooms[index].center
end

function MapLoader:create_map()
  for i = 0, self.map_tiles.w do
    for j = 0, self.map_tiles.h do
      self.tilemap:add_tile(i,j,MapTile:new())
      self.tilemap:get_tile(i,j):set_pos(i,j)
      self.tilemap:get_tile(i,j):set_type("wall")
      self.tilemap:set_base(i,j)
      self.tilemap:set_traverse(i,j, false)
    end
  end
  
  self.tilemap:clear_lit_tiles()
    
  while self.map_full == false do
    self:create_room()
  end
  
  for i = 1, #map.rooms do
    local x = self:get_roompos(i).x
    local y = self:get_roompos(i).y
    local w = self:get_roomsize(i).w
    local h = self:get_roomsize(i).h
    local center_x = self:get_roomcenter(i).x
    local center_y = self:get_roomcenter(i).y
    
    for i = x, x + w do
      for j = y, y + h do
        if self.tilemap:get_tile(i,j) then
          self.tilemap:get_tile(i,j):set_type("ground")
          self.tilemap:set_base(i,j)
          self.tilemap:set_traverse(i,j, true)
        end
      end
    end
  end
  
  local prev_room = {}
  local next_room = {}
  local doorPosition = {}
    doorPosition.x = nil
    doorPosition.y = nil
  
  for i = 1, #map.rooms - 1 do
    map.rooms[i]:place_doors()
    prev_room.x = map.rooms[i].center.x
    prev_room.y = map.rooms[i].center.y
    next_room.x = map.rooms[i + 1].center.x
    next_room.y = map.rooms[i + 1].center.y

    local tiles = {}

    for x = (prev_room.x), next_room.x do
      if self.tilemap:get_tile(x, prev_room.y) ~= nil then
        self.tilemap:get_tile(x,prev_room.y):set_type("ground")
        self.tilemap:set_base(x,prev_room.y)
        self.tilemap:set_traverse(x,prev_room.y, true)
      end
    end
    
    if prev_room.y > next_room.y then
      for y = next_room.y, (prev_room.y ) do
        if self.tilemap:get_tile(next_room.x, y) ~= nil then
          self.tilemap:get_tile(next_room.x,y):set_type("ground")
          self.tilemap:set_base(next_room.x,y)
          self.tilemap:set_traverse(next_room.x,y, true)
        end
      end
    else 
      if prev_room.y < next_room.y then
        for y = prev_room.y, (next_room.y ) do
          self.tilemap:get_tile(next_room.x, y):set_type("ground")
          self.tilemap:set_base(next_room.x,y)
          self.tilemap:set_traverse(next_room.x,y, true)
        end
      end
    end
  end
  
  self.tilemap:create_stairs()
  
  self.enemy_spawner = EnemySpawner:new(self.tilemap, 5)
  self.item_generator = ItemGenerator:new(self.tilemap, 5)

end




