require("Lib.30log")
require("Lib.lib")
require("Door")

Room = class("Room", {map})
function Room:init(map)
  self.map = map
  
  self.pos = {}
    self.pos.x = nil
    self.pos.y = nil
  
  self.size = {}
    self.size.w = nil
    self.size.h = nil
  
  self.center = {}
    self.center.x = nil
    self.center.y = nil
    
  self.boundary = {}
  
  self.borders = {}
    self.borders.top = {}
    self.borders.bottom = {}
    self.borders.left = {}
    self.borders.right = {}
  
  self.contents = {}
end

function Room:place_doors()
  local top = self.map:get_tile(self.center.x, self.pos.y - 1)
  local top_adj = self:get_adjoining(top)
  if top_adj.viable == true then
    if top_adj.up == "ground" and top_adj.down == "ground" then
      if top_adj.left == "wall" and top_adj.right == "wall" then
        Door:new(top)
      end
    end
  end
  
  local bottom = self.map:get_tile(self.center.x, self.pos.y + self.size.h + 1)
  local bot_adj = self:get_adjoining(bottom)
  if bot_adj.viable == true then
    if bot_adj.up == "ground" and bot_adj.down == "ground" then
      if bot_adj.left == "wall" and bot_adj.right == "wall" then
        Door:new(bottom)
      end
    end
  end
  
  local left = self.map:get_tile(self.pos.x - 1, self.center.y)
  local left_adj = self:get_adjoining(left)
  if left_adj.viable == true then
    if left_adj.up == "ground" and left_adj.down == "ground" then
      if left_adj.left == "wall" and left_adj.right == "wall" then
        Door:new(left)
      end
    end
  end
  
  local right = self.map:get_tile(self.pos.x + self.size.w + 1, self.center.y)
  local right_adj = self:get_adjoining(right)
  if right_adj.viable == true then
    if right_adj.up == "wall" and right_adj.down == "wall" then
      if right_adj.left == "ground" and right_adj.right == "ground" then
        Door:new(right)
      end
    end
  end
end

function Room:set_pos(start_x)
  if start_x == 0 then
    self.pos.x = math.random(1, 3)
  else
    self.pos.x = math.random(start_x, start_x + 10)
  end
  self.pos.y = math.random(1, self.map.tile_height - 10)
end

function Room:set_size()
  self.size.w = math.random(4, 10)
  self.size.h = math.random(4, 10)
end

function Room:set_center()
  self.center.x = self.pos.x + lib.Round(self.size.w / 2)
  self.center.y = self.pos.y + lib.Round(self.size.h / 2)
end

function Room:set_borders()
  for i = self.pos.x, self.pos.x + self.size.w do
    local tile = {i, self.pos.y}
    table.insert(self.borders.top, tile)
  end
  for i = self.pos.y, self.pos.y + self.size.h do
    local tile = {self.pos.x, i}
    table.insert(self.borders.left, tile)
  end
  for i = self.pos.y, self.pos.y + self.size.h do
    local tile = {self.pos.x + self.size.w, i}
    table.insert(self.borders.right, tile)
  end
  for i = self.pos.x, self.pos.x + self.size.w do
    local tile = {i, self.pos.y + self.size.h}
    table.insert(self.borders.bottom, tile)
  end
end

function Room:get_last()
  return self.pos.x + self.size.w
end

function Room:get_adjoining(tile)
    local x = tile.index.x
    local y = tile.index.y
    local adj = {}
      adj.viable = true
  
  if x ~= nil and y ~= nil then
    adj.up = self.map:get_tile(x, y - 1)
    if adj.up ~= nil then
      adj.up = adj.up.type
    else
      adj.viable = false
    end
    
    adj.down = self.map:get_tile(x, y + 1)
    if adj.down ~= nil then
      adj.down = adj.down.type
    else
      adj.viable = false
    end
    
    adj.left = self.map:get_tile(x - 1, y)
    if adj.left ~= nil then
      adj.left = adj.left.type
    else
      adj.viable = false
    end
    
    adj.right = self.map:get_tile(x + 1, y)
    if adj.right ~= nil then
      adj.right = adj.right.type
    else
      adj.viable = false
    end
  end
  return adj
end

function Room:toString()
  local string  = "\nPos: ("..self.pos.x..", "..self.pos.y..")"
  string = string.."\nSize: ("..self.size.w..", "..self.size.h..")"
  string = string.."\nCenter: ("..self.center.x..", "..self.center.y..")\n\n"
  return string
end