class = require("Lib.30log")
require("Player")

StateManager = class("StateManager")
function StateManager:init(map, gui)
  self.state = nil
  
  self.map = map
  self.gui = gui
  self.player = nil
  
  self.turn = false
end



function StateManager:set_state(state)
  self.state = state
end



function StateManager:update()
  if self.state == "Main" then
    self:update_main()
  end
end



function StateManager:draw()
  --Draw Base Tile Images
  for i = 0, self.map.map_tiles.w do
    for j = 0, self.map.map_tiles.h do
      love.graphics.draw(self.map.tilemap:get_base(i,j), i * self.map.tile_size.w, j * self.map.tile_size.h)
    end
  end
  
  --Draw Enemy Sprites
  if self.map.enemy_spawner.count > 0 then
    local enemy_list = self.map.enemy_spawner.enemies
    for i = 1, #self.map.enemy_spawner.enemies do
      if enemy_list[i].visible then
        love.graphics.draw(enemy_list[i].image, enemy_list[i].pos.x, 
                                                enemy_list[i].pos.y)
      else
        love.graphics.setColor(map.images.fog_color)
        love.graphics.draw(enemy_list[i].image, enemy_list[i].last_seen_position.x, 
                                                enemy_list[i].last_seen_position.y)
        love.graphics.setColor(255,255,255,255)
      end
    end
  end
  
  --  Draw Items
  for i = 1, #self.map.item_generator.items do
    local item = self.map.item_generator.items[i]
    if item.on_ground then
      love.graphics.draw(item.images.ground, item.pos.x * 16, item.pos.y * 16)
    end
  end
  
  --Draw Overlay
  for i = 0, self.map.map_tiles.w do
    for j = 0, self.map.map_tiles.h do
      if self.map.tilemap:get_tile(i,j).image.overlay == nil then
        --do nothing
      elseif self.map.tilemap:get_tile(i,j).image.overlay == self.map.tilemap.images.overlay.fog then
        love.graphics.setColor(self.map.images.fog_color)
        love.graphics.draw(self.map.tilemap:get_overlay(i,j), i * self.map.tile_size.w, j * self.map.tile_size.h)
        love.graphics.setColor(255,255,255,255)
      else
        love.graphics.draw(map.tilemap:get_overlay(i,j), i * self.map.tile_size.w, j * self.map.tile_size.h)
      end
    end
  end
  
  -- Resets the overlay tiles after they are drawn to the screen
  self.map.tilemap:clear_lit_tiles()
  
  --Draw Player Sprite
  love.graphics.draw(self.player.image, self.player.point.x * 16, self.player.point.y * 16)
end



function StateManager:start()
  self.map:create_map()
  self.gui:set_invent_panel("inventory")
  self.gui:set_equip_panel("equipment")
  
  self.player = Player:new("Player", "Images/Player/player.png", self)
  self.player:spawn(map.room_count, map.rooms, map.tilemap)
  self.player:gui_init(gui.panels.invent)
  
  self.gui:set_status_panel("status", "Finn", self.player.base_hp)
  
  gui:new_border(10, 50)
  
  self.state = "Main"
end



function StateManager:update_main()
  self.map.tilemap:update(self.map.tilemap)
  self.player:update(self.map.tilemap, self.map.map_tiles.w, self.map.map_tiles.h, self)
  self.map.tilemap:update(self.map.tilemap)
  self.gui:update(self.player)

  if self.player.acted == true then
    self.player.acted = false
    self.map.enemy_spawner:update(self.player)
  end
end

function StateManager:turn()
  self.turn = true
end