class = require("Lib.30log")
require("Door")
require("Lib.lib")

TileMap = class("TileMap",{tile_width, tile_height, map_height, map_width})
  function TileMap:init(tile_width, tile_height)
    self.tiles = {}
    for i = 0, tile_width do
      self.tiles[i] = {}
      for j = 0, tile_height do
        self.tiles[i][j] = 0
      end
    end
    
    self.tile_height = tile_height
    self.tile_width = tile_width
    
    self.map_height = map_height
    self.map_width = map_width
    
    self.lit_tiles = {}
    
    self.images = {}
      self.images.overlay = {}
        self.images.overlay.fog = love.graphics.newImage("Images/Tiles/fog_01.png")
        self.images.overlay.concealed = love.graphics.newImage("Images/Tiles/concealed_01.png")
      self.images.base = {}
        self.images.base.wall = love.graphics.newImage("Images/Tiles/tile_wall.png")
        self.images.base.ground = love.graphics.newImage("Images/Tiles/tile_ground.png")
        self.images.base.door_closed = love.graphics.newImage("Images/Tiles/tile_door_closed.png")
        self.images.base.door_open = love.graphics.newImage("Images/Tiles/tile_door_open.png")
        self.images.base.stairs_down = love.graphics.newImage("Images/Tiles/tile_stairs_down.png")
        self.images.base.stairs_up = love.graphics.newImage("Images/Tiles/tile_stairs_up.png")
  end
 
 
 function TileMap:add_item(x,y,item)
   table.insert(self.tiles[x][y].items, item)
 end
 
 function TileMap:get_item(x,y)
   if self.tiles[x][y]:has_item() then
      return self.tiles[x][y].items[1]
   else
      return nil
   end
 end
 
  ----Update
  function TileMap:update()

    for index, row in pairs(self.tiles) do
      for k, tile in pairs(row) do
        if tile.discovered then
          tile.image.overlay = self.images.overlay.fog
        else
          tile.image.overlay = self.images.overlay.concealed
        end
      end
    end
    
    if self.lit_tiles ~= nil then
      self:_set_all_lit()
    end
  end
  
  
  function TileMap:tile_has_door(x,y)
    if self.tiles[x][y].door then
      return true
    else
      return false
    end
  end
  

  ----Visibility Tile List Manipulation
  function TileMap:add_lit_tile(tile)
    table.insert(self.lit_tiles, tile)
  end
  
  function TileMap:clear_lit_tiles()
    self.lit_tiles = {}
  end
  
  function TileMap:_set_all_lit()
    for i = 1, #self.lit_tiles do
      self.lit_tiles[i].image.overlay = nil
    end
  end

  
  function TileMap:_set_all_unlit()
    for i = 1, #self.lit_tiles do
      self.lit_tiles[i].lit = self.images.overlay.fog
    end
  end
  
  function TileMap:_set_tile_lit(tile)
    tile.lit = true
  end
  
  function TileMap:_set_tile_unlit(tile)
    tile.lit = false
  end
  

  ----Occupied Status
  function TileMap:is_occupied(x,y)
    if self.tiles[x][y].occupied then
      return true
    else
      return false
    end
  end
  
  function TileMap:is_stair(x,y)
    if self.tiles[x][y].stair == true then
      return true
    end
    return false
  end
  
  function TileMap:create_stairs()
    local downstairs = self:get_rand_tile()
    local upstairs = self:get_rand_tile()
    
    downstairs.type = "downstairs"
    upstairs.type = "upstairs"
    downstairs.walkable = true
    downstairs.traversable = true
    upstairs.walkable = true
    upstairs.traversable = true
    self:set_base(downstairs.index.x, downstairs.index.y)
    self:set_base(upstairs.index.x, upstairs.index.y)
    
    downstairs.stair, upstairs.stair = true
  end
  
  ----Individual Tiles
  function TileMap:get_tile(x,y)
    if x >= 0 and x <= self.tile_width then
      if y >= 0 and y <= self.tile_height then
        if self.tiles[x][y] then
          return self.tiles[x][y]
        end
      end
    else
      return nil
    end
  end
  
  function TileMap:add_tile(x,y,tile)
    self.tiles[x][y] = tile
  end
  
  function TileMap:get_rand_tile()
    local finished = false
    while finished == false do
      local x = math.random(1, self.tile_height)
      local y = math.random(1, self.tile_width)
      poss_tile = self.tiles[x][y]
      if poss_tile ~= nil and poss_tile.walkable == true then
        return poss_tile
      end
    end
  end
  
  
  
  ----Entity
  function TileMap:add_entity(x,y,entity)
    if self.tiles[x][y].occupied == false then
      self.tiles[x][y]:set_occupier(entity)
    end
  end
  
  function TileMap:get_entity(x,y)
    if self.tiles[x][y].occupied then
      return self.tiles[x][y]:get_occupier()
    end
  end
  
  function TileMap:remove_ent(x,y)
    self.tiles[x][y]:remove_entity()
  end
  
  
  ----Type
  function TileMap:get_type(x,y)
    return self.tiles[x][y].type
  end 
  
  function TileMap:change_type(x,y,type)
    self.tiles[x][y]:set_type(type)
  end
  
  
  
  ----Traversing
  function TileMap:is_traversable(x,y)
    return self.tiles[x][y].walkable
  end
  
  function TileMap:set_traverse(x,y, is_traversable)
    self.tiles[x][y]:set_traversable(is_traversable)
  end
  
  
  
  
  ----Base Image
  function TileMap:set_base(x,y)
    if self.tiles[x][y].type == "ground" then
      self.tiles[x][y].image.base = self.images.base.ground
      self.tiles[x][y].walkable = true
    else
      self.tiles[x][y].image.base = self.images.base.wall
      self.tiles[x][y].walkable = false
    end
    
    if self.tiles[x][y].type == "downstairs" then
      self.tiles[x][y].image.base = self.images.base.stairs_down
    else 
      if self.tiles[x][y].type == "upstairs" then
        self.tiles[x][y].image.base = self.images.base.stairs_up
      end
    end
    
  end 
  
  function TileMap:get_base(x,y)
    return self.tiles[x][y].image.base
  end
  
  function TileMap:check_door_position(x,y)
    
  end

  ----Overlay
  function TileMap:get_overlay(x,y)
    return self.tiles[x][y].image.overlay
  end
  
  
  ----Set Tile Visble
  function TileMap:set_visible(x,y)
    self.tiles[x][y].lit = true
  end
  
  function TileMap:set_discovered(x,y)
    self.tiles[x][y].discovered = true
  end
  
  function TileMap:set_invisible(x,y)
    self.tiles[x][y].lit = false
  end
  

      